#ifndef _zxidvers_h
#define _zxidvers_h
#define ZXID_VERSION 0x000122
#define ZXID_REL "1.22"
#define ZXID_COMPILE_DATE "1412867716"
#define ZXID_REV "$Id: 1.21-18-g90bee15 20140925-192011 sampo@ $"
#endif
